import re

from django.conf import settings
from django.conf.urls import include, url
from django.views.static import serve

from wagtail.admin import urls as wagtailadmin_urls
from wagtail.core import urls as wagtail_urls
from wagtail.documents import urls as wagtaildocs_urls

urlpatterns = [
    url(r'^admin/', include(wagtailadmin_urls)),
    url(r'^documents/', include(wagtaildocs_urls)),
    url(r'', include(wagtail_urls)),
] + [
    url(r'^%s(?P<path>.*)$' % re.escape(prefix.lstrip('/')), serve, kwargs={'document_root': document_root})
    for prefix, document_root in (
        (settings.STATIC_URL, settings.STATIC_ROOT),
        (settings.MEDIA_URL, settings.MEDIA_ROOT),
    )
]



from django.forms import ModelForm
from django.forms.widgets import Widget


class OverridenWidget(Widget):
    pass


class AlternateMediaForm(ModelForm):
    class Meta:
        widgets = {
            'tags': OverridenWidget,
            'file': OverridenWidget,
            'thumbnail': OverridenWidget,
        }



from __future__ import unicode_literals

import json
import os

from django.contrib.auth import get_user_model
from django.contrib.auth.models import Group, Permission
from django.core.files.base import ContentFile
from django.test import TestCase, modify_settings
from django.test.utils import override_settings
from django.urls import reverse

from six import b
from wagtail.core.models import Collection, GroupCollectionPermission, Page
from wagtail.tests.utils import WagtailTestUtils

from wagtailmedia import models
from wagtailmedia.tests.testapp.models import EventPage, EventPageRelatedMedia


class TestMediaIndexView(TestCase, WagtailTestUtils):
    def setUp(self):
        self.login()

    def test_simple(self):
        response = self.client.get(reverse('wagtailmedia:index'))
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'wagtailmedia/media/index.html')
        self.assertContains(response, "Add audio")
        self.assertContains(response, "Add video")

    @modify_settings(INSTALLED_APPS={
        'prepend': 'wagtailmedia.tests.testextends',
    })
    def test_extends(self):
        response = self.client.get(reverse('wagtailmedia:index'))
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'wagtailmedia/media/index.html')
        self.assertNotContains(response, "Add audio")
        self.assertNotContains(response, "Add video")
        self.assertContains(response, "You shan't act")

    def test_search(self):
        response = self.client.get(reverse('wagtailmedia:index'), {'q': "Hello"})
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.context['query_string'], "Hello")

    @staticmethod
    def make_media():
        fake_file = ContentFile(b("A boring example song"))
        fake_file.name = 'song.mp3'

        for i in range(50):
            media = models.Media(
                title="Test " + str(i),
                duration=100 + i,
                file=fake_file,
                type='audio',
            )
            media.save()

    def test_pagination(self):
        self.make_media()

        response = self.client.get(reverse('wagtailmedia:index'), {'p': 2})

        # Check response
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'wagtailmedia/media/index.html')

        # Check that we got the correct page
        self.assertEqual(response.context['media_files'].number, 2)

    def test_pagination_invalid(self):
        self.make_media()

        response = self.client.get(reverse('wagtailmedia:index'), {'p': 'Hello World!'})

        # Check response
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'wagtailmedia/media/index.html')

        # Check that we got page one
        self.assertEqual(response.context['media_files'].number, 1)

    def test_pagination_out_of_range(self):
        self.make_media()

        response = self.client.get(reverse('wagtailmedia:index'), {'p': 99999})

        # Check response
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'wagtailmedia/media/index.html')

        # Check that we got the last page
        self.assertEqual(response.context['media_files'].number, response.context['media_files'].paginator.num_pages)

    def test_ordering(self):
        orderings = ['title', '-created_at']
        for ordering in orderings:
            response = self.client.get(reverse('wagtailmedia:index'), {'ordering': ordering})
            self.assertEqual(response.status_code, 200)


class TestMediaAddView(TestCase, WagtailTestUtils):
    def setUp(self):
        self.login()

    def test_action_block(self):
        with self.settings(TEMPLATES=[
            {
                'BACKEND': 'django.template.backends.django.DjangoTemplates',
                'DIRS': [
                    os.path.join(os.path.dirname(__file__), 'templates')
                ],
                'APP_DIRS': True,
                'OPTIONS': {
                    'context_processors': [
                        'django.template.context_processors.debug',
                        'django.template.context_processors.request',
                        'django.contrib.auth.context_processors.auth',
                        'django.contrib.messages.context_processors.messages',
                        'django.template.context_processors.request',
                        'wagtail.contrib.settings.context_processors.settings',
                    ],
                    'debug': True,
                },
            }
        ]):
            response = self.client.get(reverse('wagtailmedia:add', args=('audio', )))
            self.assertEqual(response.status_code, 200)
            self.assertTemplateUsed(response, 'wagtailmedia/media/add.html')
            self.assertContains(
                response,
                '<form action="/somewhere/else" method="POST" enctype="multipart/form-data" novalidate>'
            )

    def test_get_audio(self):
        response = self.client.get(reverse('wagtailmedia:add', args=('audio', )))
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'wagtailmedia/media/add.html')

        # as standard, only the root collection exists and so no 'Collection' option
        # is displayed on the form
        self.assertNotContains(response, '<label for="id_collection">')
        self.assertContains(response, 'Add audio')
        self.assertNotContains(response, 'Add audio or video')
        self.assertContains(
            response,
            '<form action="{0}" method="POST" enctype="multipart/form-data" novalidate>'.format(
                reverse('wagtailmedia:add', args=('audio',))
            ),
            count=1
        )

    def test_get_video(self):
        response = self.client.get(reverse('wagtailmedia:add', args=('video', )))
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'wagtailmedia/media/add.html')
        self.assertContains(response, 'Add video')
        self.assertNotContains(response, 'Add audio or video')
        self.assertContains(
            response,
            '<form action="{0}" method="POST" enctype="multipart/form-data" novalidate>'.format(
                reverse('wagtailmedia:add', args=('video',))
            ),
            count=1
        )

        # as standard, only the root collection exists and so no 'Collection' option
        # is displayed on the form
        self.assertNotContains(response, '<label for="id_collection">')

    def test_get_audio_or_video(self):
        response = self.client.get(reverse('wagtailmedia:add', args=('media', )))
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'wagtailmedia/media/add.html')

        self.assertNotContains(response, 'Add video')
        self.assertContains(response, 'Add audio or video')

    def test_get_audio_with_collections(self):
        root_collection = Collection.get_first_root_node()
        root_collection.add_child(name="Evil plans")

        response = self.client.get(reverse('wagtailmedia:add', args=('audio', )))
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'wagtailmedia/media/add.html')

        self.assertContains(response, '<label for="id_collection">')
        self.assertContains(response, "Evil plans")
        self.assertContains(response, 'Add audio')
        self.assertContains(
            response,
            '<form action="{0}" method="POST" enctype="multipart/form-data" novalidate>'.format(
                reverse('wagtailmedia:add', args=('audio',))
            ),
            count=1
        )

    def test_get_video_with_collections(self):
        root_collection = Collection.get_first_root_node()
        root_collection.add_child(name="Evil plans")

        response = self.client.get(reverse('wagtailmedia:add', args=('video', )))
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'wagtailmedia/media/add.html')

        self.assertContains(response, '<label for="id_collection">')
        self.assertContains(response, "Evil plans")
        self.assertContains(response, 'Add video')
        self.assertContains(
            response,
            '<form action="{0}" method="POST" enctype="multipart/form-data" novalidate>'.format(
                reverse('wagtailmedia:add', args=('video',))
            ),
            count=1
        )

    def test_post_audio(self):
        # Build a fake file
        fake_file = ContentFile(b("A boring example song"))
        fake_file.name = 'song.mp3'

        # Submit
        post_data = {
            'title': "Test media",
            'file': fake_file,
            'duration': 100,
        }
        response = self.client.post(reverse('wagtailmedia:add', args=('audio', )), post_data)

        # User should be redirected back to the index
        self.assertRedirects(response, reverse('wagtailmedia:index'))

        # Media should be created, and be placed in the root collection
        self.assertTrue(models.Media.objects.filter(title="Test media").exists())
        root_collection = Collection.get_first_root_node()

        media = models.Media.objects.get(title="Test media")
        self.assertEqual(media.collection, root_collection)
        self.assertEqual(media.type, 'audio')

    def test_post_video(self):
        # Build a fake file
        fake_file = ContentFile(b("A boring example movie"))
        fake_file.name = 'movie.mp4'

        # Submit
        post_data = {
            'title': "Test media",
            'file': fake_file,
            'duration': 100,
            'width': 720,
            'height': 480,
        }
        response = self.client.post(reverse('wagtailmedia:add', args=('video', )), post_data)

        # User should be redirected back to the index
        self.assertRedirects(response, reverse('wagtailmedia:index'))

        # Media should be created, and be placed in the root collection
        self.assertTrue(models.Media.objects.filter(title="Test media").exists())
        root_collection = Collection.get_first_root_node()
        media = models.Media.objects.get(title="Test media")
        self.assertEqual(media.collection, root_collection)
        self.assertEqual(media.type, 'video')

    def test_post_audio_with_collections(self):
        root_collection = Collection.get_first_root_node()
        evil_plans_collection = root_collection.add_child(name="Evil plans")

        # Build a fake file
        fake_file = ContentFile(b("A boring example song"))
        fake_file.name = 'song.mp3'

        # Submit
        post_data = {
            'title': "Test media",
            'file': fake_file,
            'duration': 100,
            'collection': evil_plans_collection.id,
        }
        response = self.client.post(reverse('wagtailmedia:add', args=('audio', )), post_data)

        # User should be redirected back to the index
        self.assertRedirects(response, reverse('wagtailmedia:index'))

        # Media should be created, and be placed in the Evil Plans collection
        self.assertTrue(models.Media.objects.filter(title="Test media").exists())

        media = models.Media.objects.get(title="Test media")
        self.assertEqual(media.collection, evil_plans_collection)
        self.assertEqual(media.type, 'audio')

    def test_post_video_with_collections(self):
        root_collection = Collection.get_first_root_node()
        evil_plans_collection = root_collection.add_child(name="Evil plans")

        # Build a fake file
        fake_file = ContentFile(b("A boring example movie"))
        fake_file.name = 'movie.mp3'

        # Submit
        post_data = {
            'title': "Test media",
            'file': fake_file,
            'duration': 100,
            'collection': evil_plans_collection.id,
        }
        response = self.client.post(reverse('wagtailmedia:add', args=('video', )), post_data)

        # User should be redirected back to the index
        self.assertRedirects(response, reverse('wagtailmedia:index'))

        # Media should be created, and be placed in the Evil Plans collection
        self.assertTrue(models.Media.objects.filter(title="Test media").exists())

        media = models.Media.objects.get(title="Test media")
        self.assertEqual(media.collection, evil_plans_collection)
        self.assertEqual(media.type, 'video')


class TestMediaAddViewWithLimitedCollectionPermissions(TestCase, WagtailTestUtils):
    def setUp(self):
        add_media_permission = Permission.objects.get(
            content_type__app_label='wagtailmedia', codename='add_media'
        )
        admin_permission = Permission.objects.get(
            content_type__app_label='wagtailadmin', codename='access_admin'
        )

        root_collection = Collection.get_first_root_node()
        self.evil_plans_collection = root_collection.add_child(name="Evil plans")

        conspirators_group = Group.objects.create(name="Evil conspirators")
        conspirators_group.permissions.add(admin_permission)
        GroupCollectionPermission.objects.create(
            group=conspirators_group,
            collection=self.evil_plans_collection,
            permission=add_media_permission
        )

        user = get_user_model().objects.create_user(
            username='moriarty',
            email='moriarty@example.com',
            password='password'
        )
        user.groups.add(conspirators_group)

        self.client.login(username='moriarty', password='password')

    def test_get_audio(self):
        response = self.client.get(reverse('wagtailmedia:add', args=('audio', )))
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'wagtailmedia/media/add.html')

        # user only has access to one collection, so no 'Collection' option
        # is displayed on the form
        self.assertNotContains(response, '<label for="id_collection">')
        self.assertContains(response, 'Add audio')
        self.assertContains(
            response,
            '<form action="{0}" method="POST" enctype="multipart/form-data" novalidate>'.format(
                reverse('wagtailmedia:add', args=('audio',))
            ),
            count=1
        )

    def test_get_video(self):
        response = self.client.get(reverse('wagtailmedia:add', args=('video', )))
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'wagtailmedia/media/add.html')

        # user only has access to one collection, so no 'Collection' option
        # is displayed on the form
        self.assertNotContains(response, '<label for="id_collection">')
        self.assertContains(response, 'Add video')
        self.assertContains(
            response,
            '<form action="{0}" method="POST" enctype="multipart/form-data" novalidate>'.format(
                reverse('wagtailmedia:add', args=('video',))
            ),
            count=1
        )

    def test_post_audio(self):
        # Build a fake file
        fake_file = ContentFile(b("A boring example song"))
        fake_file.name = 'song.mp3'

        # Submit
        post_data = {
            'title': "Test media",
            'file': fake_file,
            'duration': 100,
        }
        response = self.client.post(reverse('wagtailmedia:add', args=('audio', )), post_data)

        # User should be redirected back to the index
        self.assertRedirects(response, reverse('wagtailmedia:index'))

        # Media should be created with type 'audio' and in the 'evil plans' collection,
        # despite there being no collection field in the form, because that's the
        # only one the user has access to
        self.assertTrue(models.Media.objects.filter(title="Test media").exists())
        media = models.Media.objects.get(title="Test media")
        self.assertEqual(media.collection, self.evil_plans_collection)
        self.assertEqual(media.type, 'audio')

    def test_post_video(self):
        # Build a fake file
        fake_file = ContentFile(b("A boring example movie"))
        fake_file.name = 'movie.mp4'

        # Submit
        post_data = {
            'title': "Test media",
            'file': fake_file,
            'duration': 100,
        }
        response = self.client.post(reverse('wagtailmedia:add', args=('video', )), post_data)

        # User should be redirected back to the index
        self.assertRedirects(response, reverse('wagtailmedia:index'))

        # Media should be created with type 'video' and in the 'evil plans' collection,
        # despite there being no collection field in the form, because that's the
        # only one the user has access to
        self.assertTrue(models.Media.objects.filter(title="Test media").exists())
        media = models.Media.objects.get(title="Test media")
        self.assertEqual(media.collection, self.evil_plans_collection)
        self.assertEqual(media.type, 'video')


class TestMediaEditView(TestCase, WagtailTestUtils):
    def setUp(self):
        self.login()

        # Build a fake file
        fake_file = ContentFile(b("A boring example song"))
        fake_file.name = 'song.mp3'

        # Create a media to edit
        self.media = models.Media.objects.create(title="Test media", file=fake_file, duration=100)

    def test_simple(self):
        response = self.client.get(reverse('wagtailmedia:edit', args=(self.media.id,)))
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'wagtailmedia/media/edit.html')
        self.assertContains(response, "Filesize")

    @modify_settings(INSTALLED_APPS={
        'prepend': 'wagtailmedia.tests.testextends',
    })
    def test_extends(self):
        response = self.client.get(reverse('wagtailmedia:edit', args=(self.media.id,)))
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'wagtailmedia/media/edit.html')
        self.assertNotContains(response, "Filesize")
        self.assertContains(response, "sweet-style")
        self.assertContains(response, "sweet-code")
        self.assertContains(response, "sweet-form-row")
        self.assertContains(response, "sweet-stats")

    def test_post(self):
        # Build a fake file
        fake_file = ContentFile(b("A boring example song"))
        fake_file.name = 'song.mp3'

        # Submit title change
        post_data = {
            'title': "Test media changed!",
            'file': fake_file,
            'duration': 100,
        }
        response = self.client.post(reverse('wagtailmedia:edit', args=(self.media.id,)), post_data)

        # User should be redirected back to the index
        self.assertRedirects(response, reverse('wagtailmedia:index'))

        # Media title should be changed
        self.assertEqual(models.Media.objects.get(id=self.media.id).title, "Test media changed!")

    def test_with_missing_source_file(self):
        # Build a fake file
        fake_file = ContentFile(b("An ephemeral media"))
        fake_file.name = 'to-be-deleted.mp3'

        # Create a new media to delete the source for
        media = models.Media.objects.create(title="Test missing source media", file=fake_file, duration=100)
        media.file.delete(False)

        response = self.client.get(reverse('wagtailmedia:edit', args=(media.id,)), {})
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'wagtailmedia/media/edit.html')

        self.assertContains(response, 'File not found')


class TestMediaDeleteView(TestCase, WagtailTestUtils):
    def setUp(self):
        self.login()

        # Create a media to delete
        self.media = models.Media.objects.create(title="Test media", duration=100)

    def test_simple(self):
        response = self.client.get(reverse('wagtailmedia:delete', args=(self.media.id,)))
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'wagtailmedia/media/confirm_delete.html')

    def test_delete(self):
        # Submit title change
        post_data = {
            'foo': 'bar'
        }
        response = self.client.post(reverse('wagtailmedia:delete', args=(self.media.id,)), post_data)

        # User should be redirected back to the index
        self.assertRedirects(response, reverse('wagtailmedia:index'))

        # Media should be deleted
        self.assertFalse(models.Media.objects.filter(id=self.media.id).exists())

    @override_settings(WAGTAIL_USAGE_COUNT_ENABLED=True)
    def test_usage_link(self):
        response = self.client.get(reverse('wagtailmedia:delete', args=(self.media.id,)))
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'wagtailmedia/media/confirm_delete.html')
        self.assertIn('Used 0 times', str(response.content))


class TestMediaChooserView(TestCase, WagtailTestUtils):
    def setUp(self):
        self.user = self.login()

    def test_simple(self):
        response = self.client.get(reverse('wagtailmedia:chooser'))
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response['Content-Type'], 'application/json')
        json_data = json.loads(response.content.decode('utf-8'))
        self.assertSetEqual(set(json_data.keys()), {
            'html', 'step', 'error_label', 'error_message', 'tag_autocomplete_url'
        })
        self.assertTemplateUsed(response, 'wagtailmedia/chooser/chooser.html')
        self.assertEqual(json_data['step'], 'chooser')
        self.assertEqual(json_data['tag_autocomplete_url'], reverse('wagtailadmin_tag_autocomplete'))

    def test_search(self):
        response = self.client.get(reverse('wagtailmedia:chooser'), {'q': "Hello"})
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.context['query_string'], "Hello")

    @staticmethod
    def make_media():
        fake_file = ContentFile(b("A boring example song"))
        fake_file.name = 'song.mp3'

        for i in range(50):
            media = models.Media(
                title="Test " + str(i),
                duration=100 + i,
                file=fake_file,
                type='audio',
            )
            media.save()

    def test_pagination(self):
        self.make_media()

        response = self.client.get(reverse('wagtailmedia:chooser'), {'p': 2})

        # Check response
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'wagtailmedia/media/list.html')

        # Check that we got the correct page
        self.assertEqual(response.context['media_files'].number, 2)

    def test_pagination_invalid(self):
        self.make_media()

        response = self.client.get(reverse('wagtailmedia:chooser'), {'p': 'Hello World!'})

        # Check response
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'wagtailmedia/media/list.html')

        # Check that we got page one
        self.assertEqual(response.context['media_files'].number, 1)

    def test_pagination_out_of_range(self):
        self.make_media()

        response = self.client.get(reverse('wagtailmedia:chooser'), {'p': 99999})

        # Check response
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'wagtailmedia/media/list.html')

        # Check that we got the last page
        self.assertEqual(response.context['media_files'].number, response.context['media_files'].paginator.num_pages)

    def test_construct_queryset_hook_browse(self):
        media = models.Media.objects.create(
            title="Test media shown",
            duration=100,
            type='audio',
            uploaded_by_user=self.user,
        )
        models.Media.objects.create(
            title="Test media not shown",
            duration=100,
            type='audio',
        )

        def filter_media(media, request):
            return media.filter(uploaded_by_user=self.user)

        with self.register_hook('construct_media_chooser_queryset', filter_media):
            response = self.client.get(reverse('wagtailmedia:chooser'))
        self.assertEqual(len(response.context['media_files']), 1)
        self.assertEqual(response.context['media_files'][0], media)

    def test_construct_queryset_hook_search(self):
        media = models.Media.objects.create(
            title="Test media shown",
            duration=100,
            type='audio',
            uploaded_by_user=self.user,
        )
        models.Media.objects.create(
            title="Test media not shown",
            duration=100,
            type='audio',
        )

        def filter_media(media, request):
            return media.filter(uploaded_by_user=self.user)

        with self.register_hook('construct_media_chooser_queryset', filter_media):
            response = self.client.get(reverse('wagtailmedia:chooser'), {'q': 'Test'})
        self.assertEqual(len(response.context['media_files']), 1)
        self.assertEqual(response.context['media_files'][0], media)


class TestMediaChooserViewPermissions(TestCase, WagtailTestUtils):
    def setUp(self):
        add_media_permission = Permission.objects.get(
            content_type__app_label='wagtailmedia', codename='add_media'
        )
        admin_permission = Permission.objects.get(
            content_type__app_label='wagtailadmin', codename='access_admin'
        )

        self.root_collection = Collection.get_first_root_node()
        self.evil_plans_collection = self.root_collection.add_child(name="Evil plans")

        conspirators_group = Group.objects.create(name="Evil conspirators")
        conspirators_group.permissions.add(admin_permission)
        GroupCollectionPermission.objects.create(
            group=conspirators_group,
            collection=self.evil_plans_collection,
            permission=add_media_permission
        )

        user = get_user_model().objects.create_user(
            username='moriarty',
            email='moriarty@example.com',
            password='password'
        )
        user.groups.add(conspirators_group)

        fake_file = ContentFile(b("A boring song"))
        fake_file.name = 'test-song.mp3'
        media = models.Media(
                title="Test",
                duration=100,
                file=fake_file,
                type='audio',
                collection=self.root_collection,
            )
        media.save()

    def test_all_permissions_views_root_media(self):
        self.login()

        response = self.client.get(reverse('wagtailmedia:chooser'), {'collection_id': self.root_collection.id})
        self.assertIn('test-song.mp3', str(response.content))

    def test_single_collection_permissions_views_nothing(self):
        self.client.login(username='moriarty', password='password')
        response = self.client.get(reverse('wagtailmedia:chooser'), {'collection_id': self.root_collection.id})
        self.assertIn('Sorry, no media files', str(response.content))


class TestMediaChooserChosenView(TestCase, WagtailTestUtils):
    def setUp(self):
        self.login()

        # Create a media to choose
        self.media = models.Media.objects.create(title="Test media", duration=100)

    def test_simple(self):
        response = self.client.get(reverse('wagtailmedia:media_chosen', args=(self.media.id,)))
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response['Content-Type'], 'application/json')
        self.assertDictEqual(json.loads(response.content.decode('utf-8')), {
            'step': 'media_chosen',
            'result': {
                'id': self.media.id,
                'title': self.media.title,
                'edit_link': reverse('wagtailmedia:edit', args=[self.media.id],)
            }
        })


class TestMediaChooserUploadView(TestCase, WagtailTestUtils):
    def setUp(self):
        self.login()

    def test_upload_audio(self):
        response = self.client.post(
            reverse('wagtailmedia:chooser_upload', args=('audio', )), {
                'media-chooser-upload-title': 'Test audio',
                'media-chooser-upload-file': ContentFile(b('A boring example')),
                'media-chooser-upload-duration': '100',
            })

        # Check response
        self.assertEqual(response.status_code, 200)

        # Check that the audio was created
        media_files = models.Media.objects.filter(title='Test audio')
        self.assertEqual(media_files.count(), 1)

        # Test that fields are populated correctly
        media = media_files.first()
        self.assertEqual(media.type, 'audio')
        self.assertEqual(media.duration, 100)

    def test_upload_video(self):
        response = self.client.post(
            reverse('wagtailmedia:chooser_upload', args=('video', )), {
                'media-chooser-upload-title': 'Test video',
                'media-chooser-upload-file': ContentFile(b('A boring example')),
                'media-chooser-upload-duration': '100',
                'media-chooser-upload-width': '640',
                'media-chooser-upload-height': '480',
            })

        # Check response
        self.assertEqual(response.status_code, 200)

        # Check that the video was created
        media_files = models.Media.objects.filter(title='Test video')
        self.assertEqual(media_files.count(), 1)

        # Test that fields are populated correctly
        media = media_files.first()
        self.assertEqual(media.type, 'video')
        self.assertEqual(media.duration, 100)
        self.assertEqual(media.width, 640)
        self.assertEqual(media.height, 480)

    def test_upload_no_file_selected(self):
        response = self.client.post(
            reverse('wagtailmedia:chooser_upload', args=('video', )), {
                'media-chooser-upload-title': 'Test video',
            })

        # Shouldn't redirect anywhere
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'wagtailmedia/chooser/chooser.html')

        # The form should have an error
        self.assertFormError(
            response, 'uploadform', 'file', 'This field is required.')

    @override_settings(DEFAULT_FILE_STORAGE='wagtail.tests.dummy_external_storage.DummyExternalStorage')
    def test_upload_with_external_storage(self):
        response = self.client.post(
            reverse('wagtailmedia:chooser_upload', args=('video', )), {
                'media-chooser-upload-title': 'Test video',
                'media-chooser-upload-file': ContentFile(b('A boring example')),
                'media-chooser-upload-duration': '100',
            })

        # Check response
        self.assertEqual(response.status_code, 200)

        # Check that the video was created
        self.assertTrue(
            models.Media.objects.filter(title='Test video').exists())


class TestMediaFilenameProperties(TestCase):
    def setUp(self):
        self.media = models.Media(title="Test media", duration=100)
        self.media.file.save('example.mp4', ContentFile("A amazing example music video"))

        self.extensionless_media = models.Media(title="Test media", duration=101)
        self.extensionless_media.file.save('example', ContentFile("A boring example music video"))

    def test_filename(self):
        self.assertEqual('example.mp4', self.media.filename)
        self.assertEqual('example', self.extensionless_media.filename)

    def test_file_extension(self):
        self.assertEqual('mp4', self.media.file_extension)
        self.assertEqual('', self.extensionless_media.file_extension)

    def tearDown(self):
        self.media.delete()
        self.extensionless_media.delete()


class TestUsageCount(TestCase, WagtailTestUtils):
    fixtures = ['test.json']

    def setUp(self):
        self.login()

    @override_settings(WAGTAIL_USAGE_COUNT_ENABLED=True)
    def test_unused_media_usage_count(self):
        media = models.Media.objects.get(id=1)
        self.assertEqual(media.get_usage().count(), 0)

    @override_settings(WAGTAIL_USAGE_COUNT_ENABLED=True)
    def test_used_media_usage_count(self):
        media = models.Media.objects.get(id=1)
        page = EventPage.objects.get(id=3)
        event_page_related_link = EventPageRelatedMedia()
        event_page_related_link.page = page
        event_page_related_link.link_media = media
        event_page_related_link.save()
        self.assertEqual(media.get_usage().count(), 1)

    def test_usage_count_does_not_appear(self):
        media = models.Media.objects.get(id=1)
        page = EventPage.objects.get(id=3)
        event_page_related_link = EventPageRelatedMedia()
        event_page_related_link.page = page
        event_page_related_link.link_media = media
        event_page_related_link.save()
        response = self.client.get(reverse('wagtailmedia:edit',
                                           args=(1,)))
        self.assertNotContains(response, 'Used 1 time')

    @override_settings(WAGTAIL_USAGE_COUNT_ENABLED=True)
    def test_usage_count_appears(self):
        media = models.Media.objects.get(id=1)
        page = EventPage.objects.get(id=3)
        event_page_related_link = EventPageRelatedMedia()
        event_page_related_link.page = page
        event_page_related_link.link_media = media
        event_page_related_link.save()
        response = self.client.get(reverse('wagtailmedia:edit',
                                           args=(1,)))
        self.assertContains(response, 'Used 1 time')

    @override_settings(WAGTAIL_USAGE_COUNT_ENABLED=True)
    def test_usage_count_zero_appears(self):
        response = self.client.get(reverse('wagtailmedia:edit',
                                           args=(1,)))
        self.assertContains(response, 'Used 0 times')


class TestGetUsage(TestCase, WagtailTestUtils):
    fixtures = ['test.json']

    def setUp(self):
        self.login()

    def test_media_get_usage_not_enabled(self):
        media = models.Media.objects.get(id=1)
        self.assertEqual(list(media.get_usage()), [])

    @override_settings(WAGTAIL_USAGE_COUNT_ENABLED=True)
    def test_unused_media_get_usage(self):
        media = models.Media.objects.get(id=1)
        self.assertEqual(list(media.get_usage()), [])

    @override_settings(WAGTAIL_USAGE_COUNT_ENABLED=True)
    def test_used_media_get_usage(self):
        media = models.Media.objects.get(id=1)
        page = EventPage.objects.get(id=3)
        event_page_related_link = EventPageRelatedMedia()
        event_page_related_link.page = page
        event_page_related_link.link_media = media
        event_page_related_link.save()
        self.assertTrue(issubclass(Page, type(media.get_usage()[0])))

    @override_settings(WAGTAIL_USAGE_COUNT_ENABLED=True)
    def test_usage_page(self):
        media = models.Media.objects.get(id=1)
        page = EventPage.objects.get(id=3)
        event_page_related_link = EventPageRelatedMedia()
        event_page_related_link.page = page
        event_page_related_link.link_media = media
        event_page_related_link.save()
        response = self.client.get(reverse('wagtailmedia:media_usage',
                                           args=(1,)))
        self.assertContains(response, 'Christmas')

    @override_settings(WAGTAIL_USAGE_COUNT_ENABLED=True)
    def test_usage_page_no_usage(self):
        response = self.client.get(reverse('wagtailmedia:media_usage',
                                           args=(1,)))
        # There's no usage so there should be no table rows
        self.assertRegex(response.content, rb'<tbody>(\s|\n)*</tbody>')



from wagtail import VERSION as WAGTAIL_VERSION

if WAGTAIL_VERSION < (2, 5):
    from wagtail.utils.pagination import paginate
else:
    from django.core.paginator import Paginator

    DEFAULT_PAGE_KEY = 'p'

    def paginate(request, items, page_key=DEFAULT_PAGE_KEY, per_page=20):
        paginator = Paginator(items, per_page)
        page = paginator.get_page(request.GET.get(page_key))
        return paginator, page



import os

DEBUG = 'INTERACTIVE' in os.environ

WAGTAILMEDIA_ROOT = os.path.dirname(__file__)
STATIC_ROOT = os.path.join(WAGTAILMEDIA_ROOT, 'test-static')
MEDIA_ROOT = os.path.join(WAGTAILMEDIA_ROOT, 'test-media')
MEDIA_URL = '/media/'

POSTGRES_PORT = os.getenv('POSTGRES_5432_TCP_PORT', '')
if POSTGRES_PORT:
    DATABASES = {
        'default': {
            'ENGINE': 'django.db.backends.postgresql',
            'NAME': 'pgdb',
            'USER': 'pguser',
            'PASSWORD': 'pgpass',
            'HOST': 'localhost',
            'PORT': POSTGRES_PORT,
        }
    }
else:
    DATABASES = {
        'default': {
            'ENGINE': os.environ.get('DATABASE_ENGINE', 'django.db.backends.sqlite3'),
            'NAME': os.environ.get('DATABASE_NAME', 'db.sqlite3'),
            'USER': os.environ.get('DATABASE_USER', None),
            'PASSWORD': os.environ.get('DATABASE_PASS', None),
            'HOST': os.environ.get('DATABASE_HOST', None),

            'TEST': {
                'NAME': os.environ.get('DATABASE_NAME', None),
            }
        }
    }


SECRET_KEY = 'not needed'

ROOT_URLCONF = 'wagtailmedia.tests.urls'

STATIC_URL = '/static/'
STATIC_ROOT = STATIC_ROOT

STATICFILES_FINDERS = (
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
)

USE_TZ = True

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
                'django.template.context_processors.request',
                'wagtail.contrib.settings.context_processors.settings',
            ],
            'debug': True,
        },
    }
]

MIDDLEWARE = [
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

MIDDLEWARE += [
    'wagtail.core.middleware.SiteMiddleware',
    'wagtail.contrib.redirects.middleware.RedirectMiddleware',
]

INSTALLED_APPS = [
    'wagtailmedia.tests.testapp',
    'wagtailmedia',

    'taggit',

    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.staticfiles',

    'wagtail.contrib.redirects',
    'wagtail.images',
    'wagtail.users',
    'wagtail.documents',
    'wagtail.admin',
    'wagtail.core',
]


# Using DatabaseCache to make sure THAT the cache is cleared between tests.
# This prevents false-positives in some wagtail core tests where we are
# changing the 'wagtail_root_paths' key which may cause future tests to fail.

REDIS_PORT = os.getenv('REDIS_6379_TCP_PORT', '')

if REDIS_PORT:
    CACHES = {
        "default": {
            'BACKEND': 'redis_cache.RedisCache',
            'LOCATION': 'localhost:%s' % REDIS_PORT,
        },
    }
else:
    CACHES = {
        'default': {
            'BACKEND': 'django.core.cache.backends.db.DatabaseCache',
            'LOCATION': 'cache',
        }
    }

PASSWORD_HASHERS = (
    'django.contrib.auth.hashers.MD5PasswordHasher',  # don't use the intentionally slow default password hasher
)


WAGTAILSEARCH_BACKENDS = {
    'default': {
        'BACKEND': 'wagtail.search.backends.db',
    }
}

# must be set for interactive demo, copied per
# https://github.com/django/django/commit/adb96617897690b3a01e39e8297ae7d67825d2bc
ALLOWED_HOSTS = '*'

WAGTAIL_SITE_NAME = "Test Site"



from __future__ import unicode_literals

from django.contrib.auth import get_user_model
from django.contrib.auth.models import Group, Permission
from django.core.files.base import ContentFile
from django.test import TestCase
from django.urls import reverse

from six import b
from wagtail.core.models import Collection, GroupCollectionPermission
from wagtail.tests.utils import WagtailTestUtils

from wagtailmedia import models


class TestMediaPermissions(TestCase):
    def setUp(self):
        # Create some user accounts for testing permissions
        User = get_user_model()
        self.user = User.objects.create_user(username='user', email='user@email.com', password='password')
        self.owner = User.objects.create_user(username='owner', email='owner@email.com', password='password')
        self.editor = User.objects.create_user(username='editor', email='editor@email.com', password='password')
        self.editor.groups.add(Group.objects.get(name='Editors'))
        self.administrator = User.objects.create_superuser(
            username='administrator',
            email='administrator@email.com',
            password='password'
        )

        # Owner user must have the add_media permission
        self.adders_group = Group.objects.create(name='Media adders')
        GroupCollectionPermission.objects.create(
            group=self.adders_group, collection=Collection.get_first_root_node(),
            permission=Permission.objects.get(codename='add_media')
        )
        self.owner.groups.add(self.adders_group)

        # Create a media for running tests on
        self.media = models.Media.objects.create(title="Test media", duration=100, uploaded_by_user=self.owner)

    def test_administrator_can_edit(self):
        self.assertTrue(self.media.is_editable_by_user(self.administrator))

    def test_editor_can_edit(self):
        self.assertTrue(self.media.is_editable_by_user(self.editor))

    def test_owner_can_edit(self):
        self.assertTrue(self.media.is_editable_by_user(self.owner))

    def test_user_cant_edit(self):
        self.assertFalse(self.media.is_editable_by_user(self.user))


class TestEditOnlyPermissions(TestCase, WagtailTestUtils):
    def setUp(self):
        # Build a fake file
        fake_file = ContentFile(b("A boring example song"))
        fake_file.name = 'song.mp3'

        self.root_collection = Collection.get_first_root_node()
        self.evil_plans_collection = self.root_collection.add_child(name="Evil plans")
        self.nice_plans_collection = self.root_collection.add_child(name="Nice plans")

        # Create a media to edit
        self.media = models.Media.objects.create(
            title="Test media", file=fake_file, collection=self.nice_plans_collection, duration=100
        )

        # Create a user with change_media permission but not add_media
        user = get_user_model().objects.create_user(
            username='changeonly',
            email='changeonly@example.com',
            password='password'
        )
        change_permission = Permission.objects.get(
            content_type__app_label='wagtailmedia', codename='change_media'
        )
        admin_permission = Permission.objects.get(
            content_type__app_label='wagtailadmin', codename='access_admin'
        )
        self.changers_group = Group.objects.create(name='Media changers')
        GroupCollectionPermission.objects.create(
            group=self.changers_group, collection=self.root_collection,
            permission=change_permission
        )
        user.groups.add(self.changers_group)

        user.user_permissions.add(admin_permission)
        self.assertTrue(self.client.login(username='changeonly', password='password'))

    def test_get_index(self):
        response = self.client.get(reverse('wagtailmedia:index'))
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'wagtailmedia/media/index.html')

        # user should not get an "Add audio" and "Add video" buttons
        self.assertNotContains(response, "Add audio")
        self.assertNotContains(response, "Add video")

        # user should be able to see media not owned by them
        self.assertContains(response, "Test media")

    def test_search(self):
        response = self.client.get(reverse('wagtailmedia:index'), {'q': "Hello"})
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.context['query_string'], "Hello")

    def test_get_add(self):
        response = self.client.get(reverse('wagtailmedia:add', args=('audio', )))
        # permission should be denied
        self.assertRedirects(response, reverse('wagtailadmin_home'))

    def test_get_edit(self):
        response = self.client.get(reverse('wagtailmedia:edit', args=(self.media.id,)))
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'wagtailmedia/media/edit.html')

        # media can only be moved to collections you have add permission for,
        # so the 'collection' field is not available here
        self.assertNotContains(response, '<label for="id_collection">')

        # if the user has add permission on a different collection,
        # they should have option to move the media
        add_permission = Permission.objects.get(
            content_type__app_label='wagtailmedia', codename='add_media'
        )
        GroupCollectionPermission.objects.create(
            group=self.changers_group, collection=self.evil_plans_collection,
            permission=add_permission
        )
        response = self.client.get(reverse('wagtailmedia:edit', args=(self.media.id,)))
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, '<label for="id_collection">')
        self.assertContains(response, 'Nice plans')
        self.assertContains(response, 'Evil plans')

    def test_post_edit(self):
        # Submit title change
        response = self.client.post(
            reverse('wagtailmedia:edit', args=(self.media.id,)), {
                'title': "Test media changed!",
                'file': '',
                'duration': 100,
            }
        )

        # User should be redirected back to the index
        self.assertRedirects(response, reverse('wagtailmedia:index'))

        # Media title should be changed
        self.assertEqual(
            models.Media.objects.get(id=self.media.id).title,
            "Test media changed!"
        )

        # collection should be unchanged
        self.assertEqual(
            models.Media.objects.get(id=self.media.id).collection,
            self.nice_plans_collection
        )

        # if the user has add permission on a different collection,
        # they should have option to move the media
        add_permission = Permission.objects.get(
            content_type__app_label='wagtailmedia', codename='add_media'
        )
        GroupCollectionPermission.objects.create(
            group=self.changers_group, collection=self.evil_plans_collection,
            permission=add_permission
        )

        self.client.post(
            reverse('wagtailmedia:edit', args=(self.media.id,)), {
                'title': "Test media changed!",
                'collection': self.evil_plans_collection.id,
                'file': '',
                'duration': 100,
            }
        )
        self.assertEqual(
            models.Media.objects.get(id=self.media.id).collection,
            self.evil_plans_collection
        )

    def test_get_delete(self):
        response = self.client.get(reverse('wagtailmedia:delete', args=(self.media.id,)))
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'wagtailmedia/media/confirm_delete.html')



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-04-22 14:21
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import modelcluster.fields

from wagtail.core import fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('wagtailmedia', '0001_initial'),
        ('wagtailcore', '0028_merge'),
    ]

    operations = [
        migrations.CreateModel(
            name='EventPage',
            fields=[
                ('page_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='wagtailcore.Page')),
                ('date_from', models.DateField(null=True, verbose_name='Start date')),
                ('date_to', models.DateField(blank=True, help_text='Not required if event is on a single day', null=True, verbose_name='End date')),
                ('time_from', models.TimeField(blank=True, null=True, verbose_name='Start time')),
                ('time_to', models.TimeField(blank=True, null=True, verbose_name='End time')),
                ('location', models.CharField(max_length=255)),
                ('body', fields.RichTextField(blank=True)),
                ('cost', models.CharField(max_length=255)),
                ('signup_link', models.URLField(blank=True)),
            ],
            options={
                'abstract': False,
            },
            bases=('wagtailcore.page',),
        ),
        migrations.CreateModel(
            name='EventPageRelatedMedia',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sort_order', models.IntegerField(blank=True, editable=False, null=True)),
                ('title', models.CharField(help_text='Link title', max_length=255)),
                ('link_media', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to='wagtailmedia.Media')),
                ('page', modelcluster.fields.ParentalKey(on_delete=django.db.models.deletion.CASCADE, related_name='related_media', to='wagtailmedia_tests.EventPage')),
            ],
            options={
                'ordering': ['sort_order'],
                'abstract': False,
            },
        ),
    ]



from __future__ import unicode_literals

from django.core.files import File
from django.core.files.base import ContentFile
from django.test import TestCase

from six import b

from wagtailmedia import models
from wagtailmedia.forms import get_media_form


class TestMediaQuerySet(TestCase):
    def test_search_method(self):
        # Make a test media
        media = models.Media.objects.create(title="Test media file", duration=100)

        # Search for it
        results = models.Media.objects.search("Test")
        self.assertEqual(list(results), [media])

    def test_operators(self):
        aaa_media = models.Media.objects.create(title="AAA Test media", duration=100)
        zzz_media = models.Media.objects.create(title="ZZZ Test media", duration=100)

        results = models.Media.objects.search("aaa test", operator='and')
        self.assertEqual(list(results), [aaa_media])

        results = models.Media.objects.search("aaa test", operator='or')
        sorted_results = sorted(results, key=lambda media: media.title)
        self.assertEqual(sorted_results, [aaa_media, zzz_media])

    def test_custom_ordering(self):
        aaa_media = models.Media.objects.create(title="AAA Test media", duration=100)
        zzz_media = models.Media.objects.create(title="ZZZ Test media", duration=100)

        results = models.Media.objects.order_by('title').search("Test")
        self.assertEqual(list(results), [aaa_media, zzz_media])
        results = models.Media.objects.order_by('-title').search("Test")
        self.assertEqual(list(results), [zzz_media, aaa_media])

    def _test_form_init_with_non_editable_field(self, media_type, field_name):
        MediaForm = get_media_form(models.Media)
        models.Media._meta.get_field(field_name).editable = False
        media = models.Media.objects.create(title="Test media file", type=media_type, duration=100)
        try:
            MediaForm(media)
        finally:
            models.Media._meta.get_field(field_name).editable = True

    def test_form_init_with_non_editable_field(self):
        for media_type in ('audio', 'video'):
            for field_name in ('width', 'height', 'thumbnail'):
                self._test_form_init_with_non_editable_field(media_type, field_name)

    def test_audio_form_presents_thumbnail(self):
        MediaForm = get_media_form(models.Media)
        media = models.Media.objects.create(title="Test media file", type='audio', duration=100)
        self.assertIn('thumbnail', MediaForm(instance=media).fields.keys())


class TestAbstractMediaInterfaceModel(TestCase):
    def test_sources_mp4_type(self):
        fake_file = ContentFile(b("A boring example movie"))
        fake_file.name = 'movie.mp4'
        media = models.Media()
        media.file = File(fake_file)
        self.assertEqual(media.sources, [{
            'src': '/media/movie.mp4',
            'type': 'video/mp4',
        }])

    def test_sources_unknown_type(self):
        fake_file = ContentFile(b("A boring example movie"))
        fake_file.name = 'movie'
        media = models.Media()
        media.file = File(fake_file)
        self.assertEqual(media.sources, [{
            'src': '/media/movie',
            'type': 'application/octet-stream',
        }])



from django.conf.urls import url

from wagtailmedia.views import chooser, media

urlpatterns = [
    url(r'^$', media.index, name='index'),
    url(r'^(\w+)/add/$', media.add, name='add'),
    url(r'^edit/(\d+)/$', media.edit, name='edit'),
    url(r'^delete/(\d+)/$', media.delete, name='delete'),

    url(r'^chooser/$', chooser.chooser, name='chooser'),
    url(r'^chooser/(\d+)/$', chooser.media_chosen, name='media_chosen'),
    url(r'^(audio|video)/chooser/upload/$',
        chooser.chooser_upload, name='chooser_upload'),
    url(r'^usage/(\d+)/$', media.usage, name='media_usage'),
]



from django.shortcuts import get_object_or_404, render
from django.urls import reverse

from wagtail import VERSION as WAGTAIL_VERSION
from wagtail.admin.modal_workflow import render_modal_workflow
from wagtail.core import hooks
from wagtail.core.models import Collection
from wagtail.search.backends import get_search_backends

from wagtailmedia.forms import get_media_form
from wagtailmedia.models import get_media_model
from wagtailmedia.permissions import permission_policy
from wagtailmedia.utils import paginate

if WAGTAIL_VERSION < (2, 5):
    from wagtail.admin.forms import SearchForm
    pagination_template = "wagtailadmin/shared/pagination_nav.html"
else:
    from wagtail.admin.forms.search import SearchForm
    pagination_template = "wagtailadmin/shared/ajax_pagination_nav.html"

if WAGTAIL_VERSION < (2, 9):
    from wagtail.admin.utils import PermissionPolicyChecker
else:
    from wagtail.admin.auth import PermissionPolicyChecker

permission_checker = PermissionPolicyChecker(permission_policy)


def get_media_json(media):
    """
    helper function: given a media, return the json to pass back to the
    chooser panel
    """

    return {
        'id': media.id,
        'title': media.title,
        'edit_link': reverse('wagtailmedia:edit', args=(media.id,))
    }


def chooser(request):
    media_files = permission_policy.instances_user_has_any_permission_for(
        request.user, ['change', 'delete']
    )

    # allow hooks to modify the queryset
    for hook in hooks.get_hooks('construct_media_chooser_queryset'):
        media_files = hook(media_files, request)

    if permission_policy.user_has_permission(request.user, 'add'):
        Media = get_media_model()
        MediaForm = get_media_form(Media)
        uploadform = MediaForm(user=request.user, prefix='media-chooser-upload')
    else:
        uploadform = None

    q = None
    is_searching = False
    if 'q' in request.GET or 'p' in request.GET or 'collection_id' in request.GET:
        collection_id = request.GET.get('collection_id')
        if collection_id:
            media_files = media_files.filter(collection=collection_id)

        searchform = SearchForm(request.GET)
        if searchform.is_valid():
            q = searchform.cleaned_data['q']

            media_files = media_files.search(q)
            is_searching = True
        else:
            media_files = media_files.order_by('-created_at')
            is_searching = False

        # Pagination
        paginator, media_files = paginate(request, media_files, per_page=10)

        return render(request, "wagtailmedia/chooser/results.html", {
            'media_files': media_files,
            'query_string': q,
            'is_searching': is_searching,
            'pagination_template': pagination_template,
        })
    else:
        searchform = SearchForm()

        collections = Collection.objects.all()
        if len(collections) < 2:
            collections = None

        media_files = media_files.order_by('-created_at')
        paginator, media_files = paginate(request, media_files, per_page=10)

    return render_modal_workflow(request, 'wagtailmedia/chooser/chooser.html', None, {
        'media_files': media_files,
        'searchform': searchform,
        'collections': collections,
        'uploadform': uploadform,
        'is_searching': False,
        'pagination_template': pagination_template,
    }, json_data={
        'step': 'chooser',
        'error_label': "Server Error",
        'error_message': "Report this error to your webmaster with the following information:",
        'tag_autocomplete_url': reverse('wagtailadmin_tag_autocomplete'),
    })


def media_chosen(request, media_id):
    media = get_object_or_404(get_media_model(), id=media_id)

    return render_modal_workflow(
        request, None, None,
        None,
        json_data={'step': 'media_chosen', 'result': get_media_json(media)}
    )


@permission_checker.require('add')
def chooser_upload(request, media_type):
    Media = get_media_model()
    MediaForm = get_media_form(Media)

    if request.method == 'POST':
        media = Media(uploaded_by_user=request.user, type=media_type)
        form = MediaForm(
            request.POST, request.FILES, instance=media, user=request.user,
            prefix='media-chooser-upload'
        )
        if form.is_valid():
            form.save()

            # Reindex the media entry to make sure all tags are indexed
            for backend in get_search_backends():
                backend.add(media)

            return render_modal_workflow(
                request, None, None, None,
                json_data={'step': 'media_chosen',
                           'result': get_media_json(media)}
            )

    media_files = permission_policy.instances_user_has_any_permission_for(
        request.user, ['change', 'delete']
    )

    # allow hooks to modify the queryset
    for hook in hooks.get_hooks('construct_media_chooser_queryset'):
        media_files = hook(media_files, request)

    searchform = SearchForm()

    collections = Collection.objects.all()
    if len(collections) < 2:
        collections = None

    media_files = media_files.order_by('-created_at')
    paginator, media_files = paginate(request, media_files, per_page=10)

    context = {
        'media_files': media_files,
        'searchform': searchform,
        'collections': collections,
        'uploadform': form,
        'is_searching': False,
        'pagination_template': pagination_template,
        'media_type': media_type,
    }
    return render_modal_workflow(
        request, 'wagtailmedia/chooser/chooser.html', None, context,
        json_data={'step': 'chooser'}
    )



from unittest.mock import patch

from django import forms
from django.test import TestCase, override_settings

from wagtail.admin import widgets

from wagtailmedia import models
from wagtailmedia.forms import (
    BaseMediaForm, get_media_base_form, get_media_form, media_base_form
)
from wagtailmedia.tests.testapp.forms import (
    AlternateMediaForm, OverridenWidget
)


class TestFormOverride(TestCase):
    def test_media_base_form(self):
        self.assertIs(media_base_form, BaseMediaForm)

    def test_get_media_base_form(self):
        self.assertIs(get_media_base_form(), BaseMediaForm)

    def test_get_media_form(self):
        bases = get_media_form(models.Media).__bases__
        self.assertIn(BaseMediaForm, bases)
        self.assertNotIn(AlternateMediaForm, bases)

    def test_get_media_form_widgets(self):
        Form = get_media_form(models.Media)
        form = Form()
        self.assertIsInstance(form.fields['tags'].widget, widgets.AdminTagWidget)
        self.assertIsInstance(form.fields['file'].widget, forms.FileInput)
        self.assertIsInstance(form.fields['thumbnail'].widget, forms.ClearableFileInput)

    @override_settings(WAGTAILMEDIA_MEDIA_FORM_BASE='wagtailmedia.tests.testapp.forms.AlternateMediaForm')
    def test_overridden_base_form(self):
        self.assertIs(get_media_base_form(), AlternateMediaForm)

    @patch('wagtailmedia.forms.media_base_form', AlternateMediaForm)
    def test_get_overridden_media_form(self):
        bases = get_media_form(models.Media).__bases__
        self.assertNotIn(BaseMediaForm, bases)
        self.assertIn(AlternateMediaForm, bases)

    @patch('wagtailmedia.forms.media_base_form', AlternateMediaForm)
    def test_get_overridden_media_form_widgets(self):
        Form = get_media_form(models.Media)
        form = Form()
        self.assertIsInstance(form.fields['tags'].widget, OverridenWidget)
        self.assertIsInstance(form.fields['file'].widget, OverridenWidget)
        self.assertIsInstance(form.fields['thumbnail'].widget, OverridenWidget)



from django.core.files.base import ContentFile
from django.test import TestCase

from wagtailmedia.blocks import AbstractMediaChooserBlock
from wagtailmedia.models import Media


class BlockTests(TestCase):
    @classmethod
    def setUpTestData(cls):
        fake_file = ContentFile('Test')
        fake_file.name = 'test.mp3'

        cls.media = Media.objects.create(
            title='Test',
            duration=1000,
            file=fake_file,
            type='audio'
        )

    def test_abstract_render_raises_not_implemented_error(self):
        block = AbstractMediaChooserBlock()
        with self.assertRaises(NotImplementedError):
            block.render(self.media)

    def test_render_calls_render_basic(self):
        class TestMediaChooserBlock(AbstractMediaChooserBlock):
            def render_basic(self, value, context=None):
                return value.file.name

        block = TestMediaChooserBlock()
        self.assertEqual(block.render(self.media), 'media/test.mp3')
